from typing import Any

def Table(*args, **kw): ...
def Column(*args, **kw): ...

class eq_type_affinity:
    target: Any
    def __init__(self, target) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...

class eq_clause_element:
    target: Any
    def __init__(self, target) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
